searchNodes=[{"doc":"Lightweight server-side integration test functions for Phoenix. Works within the existing Phoenix.ConnTest framework and emphasizes both speed and readability. Configuration Step 1 You need to tell phoenix_integration which endpoint to use. Add the following to your phoenix application's config/test.exs file. config :phoenix_integration , endpoint : MyApp.Endpoint Where MyApp is the name of your application. Do this up before compiling phoenix_integration as part of step 2. If you change the endpoint in the config file, you will need to recompile the phoenix_integration dependency. Phoenix_integration will produce warnings if your HTML likely doesn't do what you meant. (For example, it will warn you if two text fields have the same name.) You can turn those off by adding warnings: false to the config. Step 2 Add PhoenixIntegration to the deps section of your application's mix.exs file defp deps do [ # ... { :phoenix_integration , &quot;~&gt; 0.8&quot; , only : :test } # ... ] end Don't forget to run mix deps.get Step 3 Create a test/support/integration_case.ex file. Mine simply looks like this: defmodule MyApp.IntegrationCase do use ExUnit.CaseTemplate using do quote do use MyApp.ConnCase use PhoenixIntegration end end end Alternately you could place the call to use PhoenixIntegration in your conn_case.ex file. Just make sure it is after the definition of @endpoint . Overview phoenix_integration provides two assertion and six request functions to be used alongside the existing get , post , put , patch , and delete utilities inside of a Phoenix.ConnTest test suite. The goal is to chain together a string of requests and assertions that thouroughly exercise your application in as lightweight and readable manner as possible. Each function accepts a conn and some other data, and returns a conn intended to be passed into the next function via a pipe. Examples test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;About Us&quot; ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; follow_link ( &quot;Privacy&quot; ) |&gt; follow_link ( &quot;Terms of Service&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end test &quot;Create new user&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;Sign Up&quot; ) |&gt; follow_form ( %{ user : %{ name : &quot;New User&quot; , email : &quot;user@example.com&quot; , password : &quot;test.password&quot; , confirm_password : &quot;test.password&quot; } } ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) , html : &quot;New User&quot; ) end Simulate multiple users Since all user state is held in the conn that is being passed around (just like when a user is hitting your application in a browser), you can simulate multiple users simply by tracking separate conns for them. In the example below, I'm assuming an application-specific test_sign_in function, which itself uses the follow_* functions to sign a given user in. Notice how user_conn is tracked and reused. This keeps the state the user builds up as the various links are followed, just like it would be when a proper browser is used. Example test &quot;admin grants a user permissions&quot; , %{ conn : conn , user : user , admin : admin } do # sign in the user and admin user_conn = test_sign_in ( conn , user ) admin_conn = test_sign_in ( conn , admin ) # user can&#39;t see a restricted page user_conn = get ( user_conn , page_path ( conn , :index ) ) |&gt; follow_link ( &quot;Restricted&quot; ) |&gt; assert_response ( status : 200 , path : session_path ( conn , :new ) ) |&gt; refute_response ( body : &quot;Restricted Content&quot; ) # admin grants the user permission get ( admin_conn , page_path ( conn , :index ) ) |&gt; follow_link ( &quot;Admin Dashboard&quot; ) |&gt; follow_form ( %{ user : %{ permissoin : &quot;ok_to_do_thing&quot; } } ) |&gt; assert_response ( status : 200 , path : admin_path ( conn , :index ) , html : &quot;Permission Granted&quot; ) # the user should now be able to see the restricted page get ( user_conn , page_path ( conn , :index ) ) |&gt; follow_link ( &quot;Restricted&quot; ) |&gt; assert_response ( status : 200 , path : restricted_path ( conn , :index ) , html : &quot;Restricted Content&quot; ) end Tip You can intermix IO.inspect calls in the pipe chain to help with debugging. This will print the current state of the conn into the console. test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) |&gt; follow_link ( &quot;About Us&quot; ) |&gt; IO . inspect |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end I like to use assert_response pretty heavily to make sure the content I expect is really there and to make sure I am traveling to the right locations. test &quot;Basic page flow&quot; , %{ conn : conn } do get ( conn , page_path ( conn , :index ) ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) , html : &quot;Test App&quot; ) |&gt; follow_link ( &quot;About&quot; ) |&gt; assert_response ( status : 200 , path : about_path ( conn , :index ) , html : &quot;About Test App&quot; ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; assert_response ( status : 200 , path : about_path ( conn , :contact ) , html : &quot;Contact&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) , html : &quot;Test App&quot; ) end What phoenix_integration is NOT phoenix_integration is not a client-side acceptence test suite. It does not use a real browser and does not exercise javascript code that lives there. It's focus is on fast, readable, server-side integration. Try using a tool like Hound for full-stack integration tests.","ref":"PhoenixIntegration.html","title":"PhoenixIntegration","type":"module"},{"doc":"Functions to assert/refute the response content of a conn without interrupting the chain of actions in an integration test. Each function takes a conn and a set of conditions to test. Each condition is tested and, if they all pass, the function returns the passed-in conn unchanged. If any condition fails, the function raises an appropriate error. This is intended to be used in a (possibly long) chain of piped functions that exercises a set of functionality in your application. Example test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;About Us&quot; ) |&gt; assert_response ( status : 200 , path : about_path ( conn , :index ) ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; assert_response ( content_type : &quot;text/html&quot; ) |&gt; follow_link ( &quot;Privacy&quot; ) |&gt; assert_response ( html : &quot;Privacy Policy&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end","ref":"PhoenixIntegration.Assertions.html","title":"PhoenixIntegration.Assertions","type":"module"},{"doc":"Asserts a set of conditions against the response fields of a conn. Returns the conn on success so that it can be used in the next integration call. Parameters conn should be a conn returned from a previous request should point to the path being redirected to. conditions a list of conditions to test against. Conditions can include: :status checks that conn.status equals the given numeric value :content_type the conn's content-type header should contain the given text. Typical values are &quot;text/html&quot; or &quot;application/json&quot; :body conn.resp_body should contain the given text. Does not check the content_type. :html checks that content_type is html, then looks for the given text in the body. :json checks that content_type is json, then checks that the json data equals the given map. :path the route rendered into the conn must equal the given path (or uri). :uri same as :path :redirect checks that conn.status is 302 and that the path in the &quot;location&quot; redirect header equals the given path. :to same as :redirect :assigns checks that conn.assigns contains the given values, which could be in the form of %{key =&gt; value} or [{key, value}] :value checks that the value returned by a callback (in the form fn(conn) ) is truthy Conditions can be used multiple times within a single call to assert_response . This can be useful to look for multiple text strings in the body. Example # test a rendered page assert_response ( conn , status : 200 , path : page_path ( conn , :index ) , html : &quot;Some Content&quot; , html : &quot;More Content&quot; , assigns : %{ current_user_id : user . id } ) # test a redirection assert_response ( conn , to : page_path ( conn , :index ) ) # test a callback value assert_response ( conn , value : fn ( conn ) -&gt; Guardian.Plug . current_resource ( conn ) end )","ref":"PhoenixIntegration.Assertions.html#assert_response/2","title":"PhoenixIntegration.Assertions.assert_response/2","type":"function"},{"doc":"Refutes a set of conditions for the response fields in a conn. Returns the conn on success so that it can be used in the next integration call. Parameters conn should be a conn returned from a previous request should point to the path being redirected to. conditions a list of conditions to test against. Conditions can include: :status checks that conn.status is not the given numeric value :content_type the conn's content-type header should not contain the given text. Typical values are &quot;text/html&quot; or &quot;applicaiton/json&quot; :body conn.resp_body should not contain the given text. Does not check the content_type. :html checks if content_type is html. If it is, it then checks that the given text is not in the body. :json checks if content_type is json, then checks that the json data does not equal the given map. :path the route rendered into the conn must not equal the given path (or uri). :uri same as :path :redirect checks if conn.status is 302. If it is, then checks that the path in the &quot;location&quot; redirect header is not the given path. :to same as :redirect :assigns checks that conn.assigns does not contain the given values, which could be in the form of %{key: value} or [{:key, value}] :value checks that the value returned by a callback (in the form fn(conn) ) is false or nil refute_response is often used in conjuntion with assert_response to form a complete condition check. Example # test a rendered page follow_path ( conn , page_path ( conn , :index ) ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) html : &quot;Good Content&quot; ) |&gt; refute_response ( body : &quot;Invalid Content&quot; )","ref":"PhoenixIntegration.Assertions.html#refute_response/2","title":"PhoenixIntegration.Assertions.refute_response/2","type":"function"},{"doc":"A set of functions intended to compliment the regular Phoenix.ConnTest utilities of get , post , put , patch , and delete . Each request function takes a conn and a set of data telling it what to do. Then it requests one or more paths from your phoenix application, transforming the conn each time. The final conn is returned. All the functions except follow_path and follow_redirect examine the html content of the incoming conn to find a link or form to use. In this way, you can both confirm that content exists in rendered pages and take actions as the user would. This is intended to be used as a (possibly long) chain of piped functions that exercises a set of functionality in your application. Examples test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;About Us&quot; ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; follow_link ( &quot;Privacy&quot; ) |&gt; follow_link ( &quot;Terms of Service&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end test &quot;Create new user&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # create the new user |&gt; follow_link ( &quot;Sign Up&quot; ) |&gt; follow_form ( %{ user : %{ name : &quot;New User&quot; , email : &quot;user@example.com&quot; , password : &quot;test.password&quot; , confirm_password : &quot;test.password&quot; } } ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) , html : &quot;New User&quot; ) end","ref":"PhoenixIntegration.Requests.html","title":"PhoenixIntegration.Requests","type":"module"},{"doc":"Finds a button in conn.resp_body and acts as if the user had clicked on it, and returns the resulting conn. This is very similar to click_link except that it looks for button tags as rendered by PhoenixHtml. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. identifier indicates which link to find in the html. Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. opts A map of additional options :method - method to use when requesting the path. Defaults to &quot;get&quot; ; click_button does not follow any redirects returned by the request. This allows you to explicitly check that the redirect is correct. Use follow_redirect to request the location redirected to, or just use follow_link to do it in one call. If the link is not found in the body, click_button raises an error. Examples: # click a link specified by path or uri get ( conn , thing_path ( conn , :index ) ) |&gt; click_button ( page_path ( conn , :index ) ) # click a link specified by html id with a non-get method get ( conn , thing_path ( conn , :index ) ) |&gt; click_button ( &quot;#button_id&quot; , method : :delete ) # click a link containing the given text get ( conn , thing_path ( conn , :index ) ) |&gt; click_button ( &quot;Settings&quot; ) # test a redirect and continue get ( conn , thing_path ( conn , :index ) ) |&gt; click_button ( &quot;something that redirects to new&quot; ) |&gt; assert_response ( status : 302 , to : think_path ( conn , :new ) ) |&gt; follow_redirect ( ) |&gt; assert_response ( status : 200 , path : think_path ( conn , :new ) ) Returns the transformed conn after submitting the request. Button request methods that don't use the :get method Unlike trying to click anchor tags, Phoenix always puts the method in button tags as an attribute. This means that if you want to match agains tags with a non-get method you can, but you don't really need to.","ref":"PhoenixIntegration.Requests.html#click_button/3","title":"PhoenixIntegration.Requests.click_button/3","type":"function"},{"doc":"Finds a link in conn.resp_body, requests it as if the user had clicked on it, and returns the resulting conn. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. identifier indicates which link to find in the html. Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. opts A map of additional options :method - method to use when requesting the path. Defaults to &quot;get&quot; ; click_link does not follow any redirects returned by the request. This allows you to explicitly check that the redirect is correct. Use follow_redirect to request the location redirected to, or just use follow_link to do it in one call. If the link is not found in the body, click_link raises an error. Examples: # click a link specified by path or uri get ( conn , thing_path ( conn , :index ) ) |&gt; click_link ( page_path ( conn , :index ) ) # click a link specified by html id with a non-get method get ( conn , thing_path ( conn , :index ) ) |&gt; click_link ( &quot;#link-id&quot; , method : :delete ) # click a link containing the given text get ( conn , thing_path ( conn , :index ) ) |&gt; click_link ( &quot;Settings&quot; ) # test a redirect and continue get ( conn , thing_path ( conn , :index ) ) |&gt; click_link ( &quot;something that redirects to new&quot; ) |&gt; assert_response ( status : 302 , to : think_path ( conn , :new ) ) |&gt; follow_redirect ( ) |&gt; assert_response ( status : 200 , path : think_path ( conn , :new ) ) Links that don't use the :get method When Phoenix.Html renders a link, it usually generates an &lt;a&gt; tag. However, if you specify a method other than :get, then Phoenix generates html looks like a link, but is really a form using the method. This is why you must specify the method used in opts if you used anything other than the standard :get in your link. # follow a non-get link click_link ( conn , thing_path ( conn , :delete ) , method : :delete )","ref":"PhoenixIntegration.Requests.html#click_link/3","title":"PhoenixIntegration.Requests.click_link/3","type":"function"},{"doc":"Convenience function to find and return a form in a conn.resp_body. Returns the form as a map. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. opts A map of additional options identifier indicates which link to find in the html. Defaults to nil . Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. :method - restricts the forms searched to those whose action uses the given method (such as &quot;post&quot; or &quot;put&quot;). Defaults to nil ; :finder - finding string passed to Floki.find . Defaults to &quot;form&quot; If no opts.identifier is specified, the first form that makes sense is used. Unless you have multiple forms on your page, this often is the most understandable pattern. If no appropriate form is found, fetch_form raises an error. If you have more than one form in the response, you will probably need to use the identifier options similar to what how you specify a form for submit_form or follow_form. Example: # get the value from a form on the page. fetch_form ( conn ) ## returns something like... %{ id : &quot;some_id&quot; , method : &quot;put&quot; , action : &quot;/some/action&quot; inputs : %{ user : %{ first_name : &quot;Jane&quot; , last_name : &quot;Doe&quot; } } } Note: this fetches the form as it is in the response. It will not show you updates you are making as you prepare for the next submission. In HTML, tags without values are not sent to the controller. For your convenience, this function shows you that valueless tags are in the form, marking them with :no_value . Note that some tags have values even if there's no value or checked attribute. For example, a text input's default value is the empty string.","ref":"PhoenixIntegration.Requests.html#fetch_form/2","title":"PhoenixIntegration.Requests.fetch_form/2","type":"function"},{"doc":"Finds a button in conn.resp_body, acts as if the user had clicked on it, follows any redirects, and returns the resulting conn. This is very similar to follow_link except that it looks for button tags as rendered by PhoenixHtml. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. identifier indicates which link to find in the html. Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. opts A map of additional options :method - method to use when requesting the path. Defaults to &quot;get&quot; ; :max_redirects - Maximum number of redirects to follow. Defaults to 5 ; If the link is not found in the body, follow_button raises an error. Example: # click through several pages that should point to each other get ( conn , thing_path ( conn , :index ) ) |&gt; follow_button ( &quot;#settings_button&quot; ) |&gt; follow_button ( &quot;Cancel&quot; ) |&gt; assert_response ( path : thing_path ( conn , :index ) ) Button request methods that don't use the :get method Returns the transformed conn after submitting, then following the request. Unlike trying to follow anchor tags, Phoenix always puts the method in button tags as an attribute. This means that if you want to match agains tags with a non-get method you can, but you don't really need to.","ref":"PhoenixIntegration.Requests.html#follow_button/3","title":"PhoenixIntegration.Requests.follow_button/3","type":"function"},{"doc":"Calls a function and follows the any redirects in the returned conn . If the function returns anything other than a conn , then the result is ignored and follow_fn will simply return the original conn This gives a way to insert custom assertions, or other setup code without breaking the piped chain of functions. Parameters conn A conn that has been set up to work in the test environment. Could be the conn originally passed in to the test; func a function in the form of fn(conn) -&gt; end ; opts A map of additional options :max_redirects - Maximum number of redirects to follow. Defaults to 5 ; Example: follow_fn ( conn , fn ( c ) -&gt; &quot;/some_path/&quot; &lt;&gt; token = c . request_path assert token == &quot;valid_token&quot; end )","ref":"PhoenixIntegration.Requests.html#follow_fn/3","title":"PhoenixIntegration.Requests.follow_fn/3","type":"function"},{"doc":"Finds a form in conn.resp_body, fills out the fields with the given data, requests the form's action, follows any redirects and returns the resulting conn. Similar to submit_form , except that it does follow redirects. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. fields is a map of fields and data to be written into the form before submitting its action. The data can take one of three forms: Most frequently, it's a string. It can be a list of strings. That's used when a set of tags in the form have names ending with [] to tell Phoenix to create a list value. See the example below. It can be an Elixir struct like DateTime . In that case, the fields within the struct are used to find matching tags (by name) in the form. Fields that don't match are ignored. See the example below. If you use Plug.Upload , you can set an input type=&quot;file&quot; value to the %Plug.Upload{} value you'd expect Phoenix to deliver to your controller action. See the example below. opts A map of additional options identifier indicates which link to find in the html. Defaults to nil . Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. :method - restricts the forms searched to those whose action uses the given method (such as &quot;post&quot; or &quot;put&quot;). Defaults to nil ; :finder - finding string passed to Floki.find . Defaults to &quot;form&quot; If no opts.identifier is specified, the first form that makes sense is used. Unless you have multiple forms on your page, this often is the most understandable pattern. If no appropriate form is found, follow_form raises an error. Example: upload = % Plug.Upload { content_type : &quot;image/jpg&quot; , path : &quot;/var/mytests/photo.jpg&quot; , filename : &quot;photo.jpg&quot; } # fill out a form and submit it get ( conn , thing_path ( conn , :edit , thing ) ) |&gt; follow_form ( %{ thing : %{ name : &quot;Updated Name&quot; , expires : ~D[2011-09-23] , some_count : 42 , comments : [ &quot;first&quot; , &quot;second&quot; ] , photo : upload } } ) |&gt; assert_response ( status : 200 , path : thing_path ( conn , :show , thing ) ) In this example, the form would contain list-creating HTML like this: &lt; input id = &quot;comment1&quot; type = &quot;text&quot; name = &quot;thing[comments][]&quot; value = &quot;&quot; &gt; &lt; input id = &quot;comment2&quot; type = &quot;text&quot; name = &quot;thing[comments][]&quot; value = &quot;&quot; &gt; As it happens, the form has tags for only the month and year of the expiration date: &lt; select name = &quot;thing[expires][year]&quot; &gt; ... &lt; / select &gt; &lt; select name = &quot;thing[expires][month]&quot; &gt; ... &lt; / select &gt; ... so the day part of the Date is ignored. The photo part of the form might have been created like this: &lt; % = file_input f , :photo % &gt;","ref":"PhoenixIntegration.Requests.html#follow_form/3","title":"PhoenixIntegration.Requests.follow_form/3","type":"function"},{"doc":"Finds a link in conn.resp_body, requests it as if the user had clicked on it, follows any redirects, and returns the resulting conn. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. identifier indicates which link to find in the html. Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. opts A map of additional options :method - method to use when requesting the path. Defaults to &quot;get&quot; ; :max_redirects - Maximum number of redirects to follow. Defaults to 5 ; This is similar to click_link , except that it follows returned redirects. This is very useful during integration tests as you typically want to emulate what the user is really doing. You will probably use follow_link more than click_link . If the link is not found in the body, follow_link raises an error. Example: # click through several pages that should point to each other get ( conn , thing_path ( conn , :index ) ) |&gt; follow_link ( &quot;#settings&quot; ) |&gt; follow_link ( &quot;Cancel&quot; ) |&gt; assert_response ( path : thing_path ( conn , :index ) ) Links that don't use the :get method When Phoneix.Html renders a link, it usually generates an &lt;a&gt; tag. However, if you specify a method other than :get, then Phoenix generates html looks like a link, but is really a form using the method. This is why you must specify the method used in opts if you used anything other than the standard :get in your link. # follow a non-get link follow_link ( conn , thing_path ( conn , :delete ) , method : :delete )","ref":"PhoenixIntegration.Requests.html#follow_link/3","title":"PhoenixIntegration.Requests.follow_link/3","type":"function"},{"doc":"Similar to a standard get/post/put/patch/delete call in a ConnTest except that follow_path follows any redirects returned in the conn's response header. Unlike the rest of the functions in this module, follow_path ignores the conn.resp_body and simply uses the given path. Parameters conn A conn that has been set up to work in the test environment. Could be the conn originally passed in to the test; path A path that works with your router; opts A map of additional options :method - method to use when requesting the path. Defaults to &quot;get&quot; ; :max_redirects - Maximum number of redirects to follow. Defaults to 5 ; Example: follow_path ( conn , thing_path ( conn , :index ) ) |&gt; assert_response ( status : 200 , path : think_path ( conn , :index ) )","ref":"PhoenixIntegration.Requests.html#follow_path/3","title":"PhoenixIntegration.Requests.follow_path/3","type":"function"},{"doc":"Given a conn who's response is a redirect, follow_redirect calls the path indicated by the &quot;location&quot; response header and returns the conn from that call. Parameters conn A conn whose status 302, which is a redirect. The conn's location header should point to the path being redirected to. max_redirects The maximum number of recirects to follow. Defaults to 5 ; Any incoming conn.status other than 302 causes follow_redirect to take no action and return the incoming conn for further processing.","ref":"PhoenixIntegration.Requests.html#follow_redirect/2","title":"PhoenixIntegration.Requests.follow_redirect/2","type":"function"},{"doc":"Finds a form in conn.resp_body, fills out the fields with the given data, requests the form's action and returns the resulting conn. Parameters conn should be a conn returned from a previous request that rendered some html. The functions are designed to pass the conn from one call into the next via pipes. fields a map of fields and data to be written into the form before submitting its action. opts A map of additional options identifier indicates which link to find in the html. Defaults to nil . Valid values can be in the following forms: &quot;/some/path&quot; specify the link's href starting with a &quot;/&quot; character &quot;http://www.example.com/some/uri&quot; , specify the href as full uri starting with either &quot;http&quot; or &quot;https&quot; &quot;#element-id&quot; specify the html element id of the link you are looking for. Must start start with the &quot;#&quot; character (same as css id specifier). &quot;Some Text&quot; specify text contained within the link you are looking for. :method - restricts the forms searched to those whose action uses the given method (such as &quot;post&quot; or &quot;put&quot;). Defaults to nil ; :finder - finding string passed to Floki.find . Defaults to &quot;form&quot; If no opts.identifier is specified, the first form that makes sense is used. Unless you have multiple forms on your page, this often is the most understandable pattern. If no appropriate form is found, submit_form raises an error. Any redirects are not followed. Example: # fill out a form and submit it get ( conn , thing_path ( conn , :edit , thing ) ) |&gt; submit_form ( %{ thing : %{ name : &quot;Updated Name&quot; , some_count : 42 } } ) |&gt; assert_response ( status : 302 , to : thing_path ( conn , :show , thing ) )","ref":"PhoenixIntegration.Requests.html#submit_form/3","title":"PhoenixIntegration.Requests.submit_form/3","type":"function"},{"doc":"phoenix_integration","ref":"readme.html","title":"phoenix_integration","type":"extras"},{"doc":"PhoenixIntegration is set of lightweight, server-side integration test functions for Phoenix. Works within the existing Phoenix.ConnTest framework and emphasizes both speed and readability. The goal is to chain together a string of requests and assertions that thoroughly exercise your application in as lightweight and readable manner as possible. I love the pipe |&gt; command in Elixir. By using the pipe to chain together calls in an integration test, phoenix_integration is able to be very readable. Tight integration with Phoenix.ConnTest means the calls all use the fast-path to your application for speed. Version 0.6 moves from Poison to Jason for Phoenix 1.4 compatibility. Version 0.7 requires Floki 0.24.0 or higher. Otherwise it is a patch-like update.","ref":"readme.html#overview","title":"phoenix_integration - Overview","type":"extras"},{"doc":"You can read the full documentation here .","ref":"readme.html#documentation","title":"phoenix_integration - Documentation","type":"extras"},{"doc":"Step 1 You need to tell phoenix_integration which endpoint to use. Add the following to your phoenix application's config/test.exs file. config :phoenix_integration , endpoint : MyApp.Endpoint Where MyApp is the name of your application. Do this up before compiling phoenix_integration as part of step 2. If you change the endpoint in the config file, you will need to recompile the phoenix_integration dependency. Phoenix_integration will produce warnings if your HTML likely doesn't do what you meant. (For example, it will warn you if two text fields have the same name.) You can turn those off by adding warnings: false to the config. Step 2 Add PhoenixIntegration to the deps section of your application's mix.exs file defp deps do [ # ... { :phoenix_integration , &quot;~&gt; 0.9&quot; , only : :test } # ... ] end Don't forget to run mix deps.get Step 3 Create a test/support/integration_case.ex file. Mine simply looks like this: defmodule MyApp.IntegrationCase do use ExUnit.CaseTemplate using do quote do use MyApp.ConnCase use PhoenixIntegration end end end Alternately you could place the call to use PhoenixIntegration in your conn_case.ex file. Just make sure it is after the definition of @endpoint . Step 4 Start writing integration tests. They should use your integration_conn.ex file. Here is a full example (just the name of the app is changed). This is from the location test/integration/page_integration_test.exs defmodule MyApp.AboutIntegrationTest do use MyApp.IntegrationCase , async : true test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;About Us&quot; ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; follow_link ( &quot;Privacy&quot; ) |&gt; follow_link ( &quot;Terms of Service&quot; ) |&gt; follow_button ( &quot;Accept&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end end Each function in phoenix_integration accepts a conn and some other data, and returns a conn. This conn is intended to be passed into the next function via a pipe to build up a clear, readable chain of events in your test.","ref":"readme.html#configuration","title":"phoenix_integration - Configuration","type":"extras"},{"doc":"The PhoenixIntegration.Requests module contains a set functions that make requests to your application through the router. In general, these functions look for links or forms in the html returned by a previous request. Then they make a new request to application as specified by your test. If the link wasn’t found, then an appropriate error is raised. See the full documentation for details. For example, a call such as follow_link( conn, &quot;About Us&quot; ) , looks in conn.body_request (which should contain html from a previous request), for an anchor tag that contains the visible text ‘About Us’. Note that it uses =~ and not == to look for the text, so you only need to specify enough text to find the link. These functions are also pretty flexible. A call such as follow_link( conn, &quot;/about/us&quot; ) recognizes that this is a path, so it looks for an anchor tag with an href equal to “/about/us” . Similarly, you could pass in a css-style id such as “#about_us” to find an anchor with the specified html id. Handling Redirects All functions of the form follow_* make a request to your app. Then if a redirect is returned, makes another request following the redirect. This will go on until max_redirects is reached. The goal is that (similar to Capybara), your integration test code looks like a set of actions that a user would actually do. To a user, redirects just happen. Clicking links and following forms are what is important. Submitting Forms The follow_form function finds a form in the body of the previously returned conn, fills in the fields you have specified (raising an appropriate error if the form or fields aren’t found), submits the form to your application, and follows any redirects. Used in a integration pipe chain, it looks like this: test &quot;Create new user&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) |&gt; follow_link ( &quot;Sign Up&quot; ) |&gt; follow_form ( %{ user : %{ name : &quot;New User&quot; , email : &quot;user@example.com&quot; , password : &quot;test.password&quot; , confirm_password : &quot;test.password&quot; } } ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) , html : &quot;New User&quot; ) end The submit_form function is very similar, except that you handle any redirects yourself. Tracking multiple users A very common scenario involves interactions between multiple users. The good news is that user state is returned in the conn from your controllers, so it is easy to track. Is this example, I use a test_sign_in_user function (not shown), which uses token authentication so that I don’t have to pay the BCrypt price every time I run a test… test &quot;admin can create a thing&quot; , %{ conn : conn } do # create and sign in admin admin = test_insert_user permissions : @admin_perms admin_conn = test_sign_in_user ( conn , admin ) # create and sign in regular user user = test_insert_user user_conn = test_sign_in_user ( conn , user ) # admin create a new thing get ( admin_conn , admin_path ( conn , :index ) ) |&gt; follow_link ( &quot;Create Thing&quot; ) |&gt; follow_form ( %{ thing : %{ name : &quot;New Thing&quot; } } ) |&gt; assert_response ( status : 200 , path : admin_path ( conn , :index ) , html : &quot;New Thing&quot; ) # load the thing thing = Repo . get_by ( Thing , name : &quot;New Thing&quot; ) assert thing # the user should be able to view the thing get ( user_conn , page_path ( conn , :index ) ) |&gt; follow_link ( thing . name ) |&gt; assert_response ( status : 200 , path : thing_path ( conn , :show , thing ) , html : &quot;New Thing&quot; ) end","ref":"readme.html#making-requests","title":"phoenix_integration - Making Requests","type":"extras"},{"doc":"I really wanted to see unbroken chains of piped call to make it really clear that this was a chain of events/state being tested. The following line, which is very common in Phoenix.ConnTest controller tests works well, but doesn’t allow you to build that chain of commands. assert html_response(conn, 200) =~ “Some text” So, the PhoenixIntegration.Assertions module introduces two new functions, which can test multiple conditions in a single call, and always return the (unchanged) conn being tested. See the full documentation for details. I use assert_response at almost a 1:1 ratio with the various request calls, so my tests often look something like this: test &quot;Basic page flow&quot; , %{ conn : conn } do # get the root index page get ( conn , page_path ( conn , :index ) ) # click/follow through the various about pages |&gt; follow_link ( &quot;About Us&quot; ) |&gt; assert_response ( status : 200 , path : about_path ( conn , :index ) ) |&gt; follow_link ( &quot;Contact&quot; ) |&gt; assert_response ( content_type : &quot;text/html&quot; ) |&gt; follow_link ( &quot;Privacy&quot; ) |&gt; assert_response ( html : &quot;Privacy Policy&quot; ) |&gt; follow_button ( &quot;Accept&quot; ) |&gt; assert_response ( html : &quot;Privacy Policy&quot; ) |&gt; follow_link ( &quot;Home&quot; ) |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) ) end To keep the chain clean and readable, each call to assert_response takes a conn, followed by a list of conditions to assert against. These conditions can appear multiple times in a single and will be called in the order specified. |&gt; assert_response ( status : 200 , path : page_path ( conn , :index ) html : &quot;Good Content&quot; , html : &quot;More Content&quot; ) The refute_response function is very similar in form to assert_response , except that it refutes the given conditions. I find that it is used much less frequently, and usually prove that a response doesn’t have a specific piece of content. |&gt; follow_link ( &quot;Show Thing&quot; ) |&gt; assert_response ( status : 200 , path : thing_path ( conn , :show , thing ) html : &quot;Good Content&quot; ) |&gt; refute_response ( body : &quot;Bad Content&quot; )","ref":"readme.html#asserting-responses","title":"phoenix_integration - Asserting Responses","type":"extras"},{"doc":"You can read the full documentation here .","ref":"readme.html#documentation","title":"phoenix_integration - Documentation","type":"extras"}]